@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -123,7 +123,7 @@ static inline void bdaddr_dump(int level, char *str, struct frame *frm)
 {
 	char addr[18];
 
-	ba2str(frm->ptr, addr);
+	p_ba2str(frm->ptr, addr);
 
 	p_indent(level, frm);
 	printf("%s: bdaddr %s\n", str, addr);
@@ -180,6 +180,18 @@ static inline void handle_clock_dump(int level, char *str, struct frame *frm)
 	printf("%s: handle %d clock 0x%4.4x\n", str, handle, clock);
 }
 
+static inline void radiotest_dump(int level, char *str, struct frame *frm)
+{
+	uint16_t testid;
+
+	testid = CSR_U16(frm);
+
+	p_indent(level, frm);
+	printf("%s: test id %d\n", str, testid);
+
+	raw_dump(level, frm);
+}
+
 static inline void psmemtype_dump(int level, char *str, struct frame *frm)
 {
 	uint16_t store, type;
@@ -444,6 +456,9 @@ static inline void bccmd_dump(int level, struct frame *frm)
 	case 0x400f:
 		valueless_dump(level + 1, "PS_DEFRAG_RESET", frm);
 		break;
+	case 0x4011:
+		valueless_dump(level + 1, "HOPPING_ON", frm);
+		break;
 	case 0x4012:
 		valueless_dump(level + 1, "CANCEL_PAGE", frm);
 		break;
@@ -453,6 +468,12 @@ static inline void bccmd_dump(int level, struct frame *frm)
 	case 0x481c:
 		uint16_dump(level + 1, "MAP_SCO_PCM", frm);
 		break;
+	case 0x482e:
+		uint16_dump(level + 1, "SINGLE_CHAN", frm);
+		break;
+	case 0x5004:
+		radiotest_dump(level + 1, "RADIOTEST", frm);
+		break;
 	case 0x500c:
 		psstores_dump(level + 1, "PS_CLR_STORES", frm);
 		break;
@@ -577,7 +598,7 @@ void csr_dump(int level, struct frame *frm)
 					addr, master ? "master" : "slave");
 				if (!master) {
 					char addr[18];
-					ba2str((bdaddr_t *) frm->ptr, addr);
+					p_ba2str((bdaddr_t *) frm->ptr, addr);
 					p_indent(level + 1, frm);
 					printf("bdaddr %s class "
 						"0x%2.2x%2.2x%2.2x\n", addr,
